@@ -3,8 +3,29 @@ import helium
 import util
 import timeseries as ts
 import dpath.util as dpath
+from functools import update_wrapper
+
+
+pass_service = click.make_pass_decorator(helium.Service)
+
+
+def version_option(f):
+    return click.option('--versions', type=click.Choice(['none', 'fw', 'all']),
+                        default='none',
+                        help="display version information")(f)
+
+
+def mac_option(f):
+    return click.option('--mac', is_flag=True,
+                        help="Whether the given id is a mac address")(f)
+
+
+def card_type(card_id, default):
+    return {
+        '2': 'blue',
+        '5': 'green',
+    }.get(card_id, default)
 
-pass_service=click.make_pass_decorator(helium.Service)
 
 @click.group()
 def cli():
@@ -12,6 +33,11 @@ def cli():
     """
     pass
 
+
+def _find_sensor_id(service, sensor, **kwargs):
+    return util.lookup_resource_id(service.get_sensors, sensor, **kwargs)
+
+
 def _tabulate(result, **kwargs):
     version_map = []
     version_base = [
@@ -30,23 +56,28 @@ def _tabulate(result, **kwargs):
             ('config', 'meta/versions/sensor-config'),
             ('script', _map_script_versions)
         ]
+    def _map_card(json):
+        key = unicode(dpath.get(json, 'meta/card/id'))
+        return card_type(key, key)
 
     util.tabulate(result, [
         ('id', util.shorten_json_id),
         ('mac', 'meta/mac'),
+        ('type', _map_card),
+        ('seen', 'meta/last-seen'),
     ] + version_map + [
         ('name', 'attributes/name'),
     ])
 
 
 @cli.command()
-@click.option('-l', '--label',
+@click.argument('sensor', required=False)
+@click.option('-l', '--label', metavar='LABEL',
               help="the id of a label")
-@click.option('--versions', type=click.Choice(['none', 'fw', 'all']),
-              default='none',
-              help="display sensor version information")
+@version_option
+@mac_option
 @pass_service
-def list(service, label, **kwargs):
+def list(service, sensor, label, **kwargs):
     """List sensors.
 
     Lists information for a label of sensors or all sensors in
@@ -54,10 +85,13 @@ def list(service, label, **kwargs):
     """
     if label:
         label = util.lookup_resource_id(service.get_labels, label)
-        sensors = service.get_label(label,include="sensor").get('included')
+        sensors = service.get_label(label, include="sensor").get('included')
+    elif sensor:
+        sensor = _find_sensor_id(service, sensor, **kwargs)
+        sensors = [service.get_sensor(sensor).get('data')]
     else:
         sensors = service.get_sensors().get('data')
-        _tabulate(sensors, **kwargs)
+    _tabulate(sensors, **kwargs)
 
 
 @cli.command()
@@ -72,6 +106,7 @@ def create(service, name):
     sensors = [service.create_sensor(name=name).get('data')]
     _tabulate(sensors)
 
+
 @cli.command()
 @click.argument('sensor')
 @click.option('--name',
@@ -100,30 +135,42 @@ def delete(service, sensor):
     click.echo("Deleted" if result.status_code == 204 else result)
 
 
-@cli.command()
 @click.argument('sensor')
-@ts.options()
 @pass_service
-def timeseries(service, sensor, **kwargs):
-    """List readings for a sensor.
+def _get_sensor_timeseries(service, sensor, **kwargs):
+    """Get timeseries readings for a sensor.
 
-    Lists one page of readings or aggregations of readings for a given SENSOR.
+    Retrieve timeseries data for a given SENSOR.
     """
     sensor = util.lookup_resource_id(service.get_sensors, sensor)
-    data = service.get_sensor_timeseries(sensor, **kwargs).get('data')
-    ts.tabulate(data, **kwargs)
+    return service.get_sensor_timeseries(sensor, **kwargs).get('data')
+
+
+@click.argument('sensor')
+@pass_service
+def _post_sensor_timeseries(service, sensor, **kwargs):
+    """Post readings to a sensor.
+
+    Posts timeseries readings for a given SENSOR.
+    """
+    sensor = util.lookup_resource_id(service.get_sensors, sensor)
+    return [service.post_sensor_timeseries(sensor, **kwargs).get('data')]
+
+
+cli.add_command(ts.cli(get=_get_sensor_timeseries,
+                       post=_post_sensor_timeseries))
 
 
 @cli.command()
-@ts.format_option()
 @click.argument('sensor')
-@ts.options(page_size=5000)
+@ts.options(page_size=5000, format='csv')
 @pass_service
 def dump(service, sensor, format, **kwargs):
     """Dumps timeseries data to files.
 
     Dumps the timeseries data for one SENSOR to a file.
-    If no sensors or label is specified all sensors for the organization are dumped.
+    If no sensors or label is specified all sensors for the organization
+    are dumped.
 
     One file is generated with the sensor id as filename and the
     file extension based on the requested dump format
